// SPDX-FileCopyrightText: 2023 Aravinth Manivannan <realaravinth@batsense.net>
//
// SPDX-License-Identifier: AGPL-3.0-or-later

use std::collections::HashMap;

use semver::Version;
use url::Url;

use serde::{Deserialize, Serialize};

#[derive(Clone, Debug, Eq, PartialEq, Deserialize, Serialize)]
pub struct Suite {
    pub name: String,
    pub description: String,
    pub version: Version,
    pub tests: Vec<Test>,
}

#[derive(Clone, Debug, Eq, PartialEq, Deserialize, Serialize)]
pub struct Test {
    pub name: String,
    pub url: Url,
    pub version: Version,
    // URL to the container image, the version should be the tag
    pub container: String,
    pub env_vars: Option<HashMap<String, String>>,
}
